@@ -93,4 +93,16 @@ class GrandchildIndexable(ChildIndexable):
     @classmethod
     def get_serializer_class(cls):
         from .serializers import GrandchildIndexableSerializer
-        return GrandchildIndexableSerializer
\ No newline at end of file
+        return GrandchildIndexableSerializer
+
+
+class PolyMixin(PolymorphicIndexable, models.Model):
+    itsa = models.TextField(default="", blank=True)
+    mixin = models.TextField(default="", blank=True)
+
+    class Meta:
+        abstract = True
+
+
+class MixedIndexable(SeparateIndexable, PolyMixin):
+    pass
